@@ -5,6 +5,7 @@ from __future__ import absolute_import
 
 import os
 import sys
+import base64
 import shutil
 import tempfile
 import unittest
@@ -19,7 +20,7 @@ from bitarray.test_bitarray import Util
 from bitarray.util import (
     zeros, urandom, pprint, make_endian, rindex, strip, count_n,
     parity, count_and, count_or, count_xor, subset,
-    serialize, deserialize, ba2hex, hex2ba,
+    serialize, deserialize, ba2hex, hex2ba, ba2base, base2ba,
     ba2int, int2ba, huffman_code
 )
 
@@ -46,6 +47,10 @@ class TestsZeros(unittest.TestCase):
 
             for n in range(100):
                 a = zeros(n)
+                self.assertEqual(len(a), n)
+                self.assertFalse(a.any())
+                self.assertEqual(a.count(1), 0)
+                self.assertEqual(a, n * bitarray('0'))
                 self.assertEqual(a, bitarray(n * '0'))
 
             for endian in 'big', 'little':
@@ -191,6 +196,7 @@ class TestsMakeEndian(unittest.TestCase, Util):
         c = make_endian(a, 'little')
         self.assertTrue(c == a)
         self.assertEqual(c.endian(), 'little')
+        self.assertIsType(c, 'bitarray')
 
         # wrong arguments
         self.assertRaises(TypeError, make_endian, '', 'big')
@@ -211,6 +217,7 @@ class TestsMakeEndian(unittest.TestCase, Util):
         c = make_endian(a, 'little')
         self.assertTrue(c == a)
         self.assertEqual(c.endian(), 'little')
+        #self.assertIsType(c, 'frozenbitarray')
 
     def test_random(self):
         for a in self.randombitarrays():
@@ -272,19 +279,19 @@ class TestsRIndex(unittest.TestCase, Util):
                 j = None
             self.assertEqual(i, j)
 
-    def test_3(self):
-        for _ in range(100):
+    def test_many_set(self):
+        for _ in range(10):
             n = randint(1, 100000)
             v = randint(0, 1)
             a = bitarray(n)
-            a.setall(1 - v)
+            a.setall(not v)
             lst = [randint(0, n - 1) for _ in range(100)]
             for i in lst:
                 a[i] = v
             self.assertEqual(rindex(a, v), max(lst))
 
     def test_one_set(self):
-        for _ in range(100):
+        for _ in range(10):
             N = randint(1, 10000)
             a = bitarray(N)
             a.setall(0)
@@ -308,28 +315,52 @@ class TestsStrip(unittest.TestCase, Util):
             self.assertEQUAL(strip(a, 'left'), bitarray('10110000'))
             self.assertEQUAL(strip(a, 'both'), bitarray('1011'))
             b = frozenbitarray('00010110000')
-            self.assertEqual(strip(b, 'both'), bitarray('1011'))
+            c = strip(b, 'both')
+            self.assertEqual(c, bitarray('1011'))
+            self.assertIsType(c, 'frozenbitarray')
 
-        for mode in 'left', 'right', 'both':
-            self.assertEqual(strip(bitarray('000'), mode), bitarray())
-            self.assertEqual(strip(bitarray(), mode), bitarray())
+    def test_zeros(self):
+        for n in range(10):
+            for mode in 'left', 'right', 'both':
+                a = zeros(n)
+                c = strip(a, mode)
+                self.assertIsType(c, 'bitarray')
+                self.assertEqual(c, bitarray())
+                self.assertEqual(a, zeros(n))
+
+                b = frozenbitarray(a)
+                c = strip(b, mode)
+                self.assertIsType(c, 'frozenbitarray')
+                self.assertEqual(c, bitarray())
 
     def test_random(self):
         for a in self.randombitarrays():
             b = a.copy()
+            f = frozenbitarray(a)
             s = a.to01()
-            self.assertEqual(strip(a, 'left'), bitarray(s.lstrip('0')))
-            self.assertEqual(strip(a, 'right'), bitarray(s.rstrip('0')))
-            self.assertEqual(strip(a, 'both'), bitarray(s.strip('0')))
-            self.assertEQUAL(a, b)
+            for mode, res in [
+                    ('left',  bitarray(s.lstrip('0'), a.endian())),
+                    ('right', bitarray(s.rstrip('0'), a.endian())),
+                    ('both',  bitarray(s.strip('0'),  a.endian())),
+            ]:
+                c = strip(a, mode)
+                self.assertEQUAL(c, res)
+                self.assertIsType(c, 'bitarray')
+                self.assertEQUAL(a, b)
+
+                c = strip(f, mode)
+                self.assertEQUAL(c, res)
+                self.assertIsType(c, 'frozenbitarray')
+                self.assertEQUAL(f, b)
 
     def test_one_set(self):
-        for _ in range(100):
-            N = randint(1, 10000)
-            a = bitarray(N)
+        for _ in range(10):
+            n = randint(1, 10000)
+            a = bitarray(n)
             a.setall(0)
-            a[randint(0, N - 1)] = 1
+            a[randint(0, n - 1)] = 1
             self.assertEqual(strip(a, 'both'), bitarray('1'))
+            self.assertEqual(len(a), n)
 
 tests.append(TestsStrip)
 
@@ -700,6 +731,157 @@ tests.append(TestsHexlify)
 
 # ---------------------------------------------------------------------------
 
+class TestsBase(unittest.TestCase, Util):
+
+    def test_ba2base(self):
+        c = ba2base(16, bitarray('1101', 'big'))
+        self.assertIsInstance(c, str)
+
+    def test_base2ba(self):
+        _set_default_endian('big')
+        for c in 'e', 'E', b'e', b'E', u'e', u'E':
+            a = base2ba(16, c)
+            self.assertEqual(a.to01(), '1110')
+            self.assertEqual(a.endian(), 'big')
+
+    def test_explicit(self):
+        data = [ #              n  little   big
+            ('',                2, '',      ''),
+            ('1 0 1',           2, '101',   '101'),
+            ('11 01 00',        4, '320',   '310'),
+            ('111 001',         8, '74',    '71'),
+            ('1111 0001',      16, 'f8',    'f1'),
+            ('11111 00001',    32, '7Q',    '7B'),
+            ('111111 000001',  64, '/g',    '/B'),
+        ]
+        for bs, n, s_le, s_be in data:
+            a_le = bitarray(bs, 'little')
+            a_be = bitarray(bs, 'big')
+            self.assertEQUAL(base2ba(n, s_le, 'little'), a_le)
+            self.assertEQUAL(base2ba(n, s_be, 'big'),    a_be)
+            self.assertEqual(ba2base(n, a_le), s_le)
+            self.assertEqual(ba2base(n, a_be), s_be)
+
+    def test_empty(self):
+        for n in 2, 4, 8, 16, 32, 64:
+            a = base2ba(n, '')
+            self.assertEqual(a, bitarray())
+            self.assertEqual(ba2base(n, a), '')
+
+    def test_upper(self):
+        self.assertEqual(base2ba(16, 'F'), bitarray('1111'))
+
+    def test_invalid_characters(self):
+        for n, s in ((2, '2'), (4, '4'), (8, '8'), (16, 'g'), (32, '8'),
+                     (32, '1'), (32, 'a'), (64, '-'), (64, '_')):
+            self.assertRaises(ValueError, base2ba, n, s)
+
+    def test_invalid_args(self):
+        a = bitarray()
+        self.assertRaises(TypeError, ba2base, None, a)
+        self.assertRaises(TypeError, base2ba, None, '')
+        for i in range(-10, 260):
+            if i in (2, 4, 8, 16, 32, 64):
+                continue
+            self.assertRaises(ValueError, ba2base, i, a)
+            self.assertRaises(ValueError, base2ba, i, '')
+
+    def test_binary(self):
+        a = base2ba(2, '1011')
+        self.assertEqual(a, bitarray('1011'))
+        self.assertEqual(ba2base(2, a), '1011')
+
+        for a in self.randombitarrays():
+            s = ba2base(2, a)
+            self.assertEqual(s, a.to01())
+            self.assertEQUAL(base2ba(2, s, a.endian()), a)
+
+    def test_quaternary(self):
+        a = base2ba(4, '0123', 'big')
+        self.assertEqual(a, bitarray('00 01 10 11'))
+        self.assertEqual(ba2base(4, a), '0123')
+
+    def test_octal(self):
+        a = base2ba(8, '0147', 'big')
+        self.assertEqual(a, bitarray('000 001 100 111'))
+        self.assertEqual(ba2base(8, a), '0147')
+
+    def test_hexadecimal(self):
+        a = base2ba(16, 'F61', 'big')
+        self.assertEqual(a, bitarray('1111 0110 0001'))
+        self.assertEqual(ba2base(16, a), 'f61')
+
+        for n in range(100):
+            s = ''.join(choice(hexdigits) for _ in range(n))
+            for endian in 'big', 'little':
+                a = base2ba(16, s, endian)
+                self.assertEQUAL(a, hex2ba(s, endian))
+                self.assertEqual(ba2base(16, a), ba2hex(a))
+
+    def test_base32(self):
+        a = base2ba(32, '7SH', 'big')
+        self.assertEqual(a, bitarray('11111 10010 00111'))
+        self.assertEqual(ba2base(32, a), '7SH')
+
+        msg = os.urandom(randint(10, 100) * 5)
+        s = base64.b32encode(msg).decode()
+        a = base2ba(32, s, 'big')
+        self.assertEqual(a.tobytes(), msg)
+        self.assertEqual(ba2base(32, a), s)
+
+    def test_base64(self):
+        a = base2ba(64, '/jH', 'big')
+        self.assertEqual(a, bitarray('111111 100011 000111'))
+        self.assertEqual(ba2base(64, a), '/jH')
+
+        msg = os.urandom(randint(10, 100) * 3)
+        s = base64.standard_b64encode(msg).decode()
+        a = base2ba(64, s, 'big')
+        self.assertEqual(a.tobytes(), msg)
+        self.assertEqual(ba2base(64, a), s)
+
+    def test_alphabets(self):
+        for m, n, alpabet in [
+                (1,  2, '01'),
+                (2,  4, '0123'),
+                (3,  8, '01234567'),
+                (4, 16, '0123456789abcdef'),
+                (5, 32, 'ABCDEFGHIJKLMNOPQRSTUVWXYZ234567'),
+                (6, 64, 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
+                        'abcdefghijklmnopqrstuvwxyz0123456789+/'),
+        ]:
+            self.assertEqual(1 << m, n)
+            self.assertEqual(len(alpabet), n)
+            for i, c in enumerate(alpabet):
+                for endian in 'big', 'little':
+                    self.assertEqual(ba2int(base2ba(n, c, endian)), i)
+                    self.assertEqual(ba2base(n, int2ba(i, m, endian)), c)
+
+    def test_random(self):
+        for a in self.randombitarrays():
+            for m in range(1, 7):
+                n = 1 << m
+                if len(a) % m == 0:
+                    s = ba2base(n, a)
+                    b = base2ba(n, s, a.endian())
+                    self.assertEQUAL(a, b)
+                    self.check_obj(b)
+                else:
+                    self.assertRaises(ValueError, ba2base, n, a)
+
+    def test_random2(self):
+        for m in range(1, 7):
+            n = 1 << m
+            for length in range(0, 100, m):
+                a = urandom(length, 'little')
+                self.assertEQUAL(base2ba(n, ba2base(n, a), 'little'), a)
+                b = bitarray(a, 'big')
+                self.assertEQUAL(base2ba(n, ba2base(n, b), 'big'), b)
+
+tests.append(TestsBase)
+
+# ---------------------------------------------------------------------------
+
 class TestsIntegerization(unittest.TestCase, Util):
 
     def test_ba2int(self):
@@ -888,6 +1070,18 @@ class MixedTests(unittest.TestCase, Util):
             self.assertEqual(t, s)
             self.assertEqual(eval(t), i)
 
+    def test_oct(self):
+        if sys.version_info[0] == 2:
+            return
+        for i in range(1000):
+            s = oct(i)
+            self.assertEqual(s[:2], '0o')
+            a = base2ba(8, s[2:], 'big')
+            self.assertEqual(ba2int(a), i)
+            t = '0o%s' % ba2base(8, a)
+            self.assertEqual(t, s)
+            self.assertEqual(eval(t), i)
+
     def test_hex(self):
         for i in range(1000):
             s = hex(i)
@@ -898,6 +1092,52 @@ class MixedTests(unittest.TestCase, Util):
             self.assertEqual(t, s)
             self.assertEqual(eval(t), i)
 
+    def test_bitwise(self):
+        for a in self.randombitarrays(start=1):
+            b = urandom(len(a), a.endian())
+            aa = a.copy()
+            bb = b.copy()
+            i = ba2int(a)
+            j = ba2int(b)
+            self.assertEqual(ba2int(a & b), i & j)
+            self.assertEqual(ba2int(a | b), i | j)
+            self.assertEqual(ba2int(a ^ b), i ^ j)
+
+            n = randint(0, len(a))
+            if a.endian() == 'big':
+                self.assertEqual(ba2int(a >> n), i >> n)
+                c = zeros(len(a)) + a
+                self.assertEqual(ba2int(c << n), i << n)
+
+            self.assertEQUAL(a, aa)
+            self.assertEQUAL(b, bb)
+
+    def test_bitwise_inplace(self):
+        for a in self.randombitarrays(start=1):
+            b = urandom(len(a), a.endian())
+            bb = b.copy()
+            i = ba2int(a)
+            j = ba2int(b)
+            c = a.copy()
+            c &= b
+            self.assertEqual(ba2int(c), i & j)
+            c = a.copy()
+            c |= b
+            self.assertEqual(ba2int(c), i | j)
+            c = a.copy()
+            c ^= b
+            self.assertEqual(ba2int(c), i ^ j)
+            self.assertEQUAL(b, bb)
+
+            n = randint(0, len(a))
+            if a.endian() == 'big':
+                c = a.copy()
+                c >>= n
+                self.assertEqual(ba2int(c), i >> n)
+                c = zeros(len(a)) + a
+                c <<= n
+                self.assertEqual(ba2int(c), i << n)
+
 tests.append(MixedTests)
 
 # ---------------------------------------------------------------------------
